"""
A formulation in original variables of a wedding seating problem

Authors: Stuart Mitchell 2010
"""
from time import time

import pulp

def happiness(guest_a, guest_b):
    """
    Return the happiness (0 is the best) of allocating two
    guests together in the same table
    """
    return abs(ord(guest_a) - ord(guest_b))

def solve_problem(guests, max_table_size, max_tables):
    #time this application
    start_time = time()
    #create the set of possible tables
    tables = range(max_tables)

    possible_seatings = [(g, t) for g in guests
                                for t in tables]

    #create a binary variable to model if a guest sits at a particular table
    x = pulp.LpVariable.dicts('possible_seatings', possible_seatings, 
                                lowBound = 0,
                                upBound = 1,
                                cat = pulp.LpInteger)

    seating_model = pulp.LpProblem("Wedding Seating Model (DIP)", pulp.LpMinimize)

    #specify the maximum number of guests per table
    for table in tables:
        seating_model += sum([x[(guest, table)] for guest in guests]) <= max_table_size, \
                                "Maximum_table_size_%s"%table

    #A guest must seated at one and only one table
    for guest in guests:
        seating_model += sum([x[(guest, table)] for table in tables]) == 1, "Must_seat_%s"%guest

    #create a set of variables to model the objective function
    possible_pairs = [(a, b) for a in guests for b in guests if ord(a) < ord(b)]
    happy = pulp.LpVariable.dicts('table_happiness', tables,
                                  lowBound = 0,
                                  upBound = None,
                                  cat = pulp.LpContinuous)

    seating_model += sum([happy[table] for table in tables])

    #create constraints for each possible pair
    for (a, b) in possible_pairs:
        for table in tables:
            seating_model += happy[table] >= happiness(a, b) * (x[(a, table)] + x[(b, table)] - 1)

    seating_model.writeLP('wedding_orig.lp')
    seating_model.solve(pulp.GUROBI())

    end_time = time()-start_time
    return end_time, seating_model.objective.value(), tables, x, happy

if __name__ == '__main__':
    end_time, obj, tables, x, happy = solve_problem(guests, max_table_size, max_tables)
    for table in tables:
        print table,
        for guest in guests:
            if x[(guest,table)].value() >= 0.99:
                print guest,
        print happy[table].value()
        print
    print 'Total solution time for original variables %s seconds'%end_time
